@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -83,7 +83,8 @@ dispatch_file_entry_p_by_size(struct file_entry **file_entry_p,
         /* assign it */
         file_entry_p[i]->partition_index = smallest_partition_index;
 #if defined(DEBUG)
-        fprintf(stderr, "%s(): %s added to partition %ju (%p)\n", __func__,
+        fprintf(stderr, "%s(): %s assigned to partition_index %ju (%p)\n",
+            __func__,
             file_entry_p[i]->path, file_entry_p[i]->partition_index,
             smallest_partition);
 #endif
@@ -156,7 +157,7 @@ dispatch_empty_file_entries(struct file_entry *head, fnum_t num_entries,
                     /* assign new index to file entry */
                     head->partition_index = j;
 #if defined(DEBUG)
-                    fprintf(stderr, "%s(): %s (empty) re-assigned to partition "
+                    fprintf(stderr, "%s(): %s (empty) re-assigned to partition_index "
                         "%ju (%p)\n", __func__, head->path,
                         head->partition_index, part_head);
 #endif
@@ -179,7 +180,7 @@ dispatch_empty_file_entries(struct file_entry *head, fnum_t num_entries,
    - must be called with *part_head == NULL (will create partitions)
    - if max_size > 0, partition 0 will hold files that cannot be held by other
      partitions
-   - returns the number of parts created with part_head set to the last
+   - returns the number of parts created with part_head set to the first
      element */
 pnum_t
 dispatch_file_entries_by_limits(struct file_entry *head,
@@ -209,6 +210,7 @@ dispatch_file_entries_by_limits(struct file_entry *head,
     /* create a first data partition and keep a pointer to it */
     if(add_partitions(part_head, 1, options) != 0) {
         fprintf(stderr, "%s(): cannot create partition\n", __func__);
+        *part_head = default_partition;
         return (num_parts_created);
     }
     num_parts_created++;
@@ -226,7 +228,7 @@ dispatch_file_entries_by_limits(struct file_entry *head,
             default_partition->size += head->size;
             default_partition->num_files++;
 #if defined(DEBUG)
-            fprintf(stderr, "%s(): %s added to partition %ju (%p)\n",
+            fprintf(stderr, "%s(): %s assigned to partition_index %ju (%p)\n",
                 __func__, head->path, head->partition_index, default_partition);
 #endif
         }
@@ -241,6 +243,7 @@ dispatch_file_entries_by_limits(struct file_entry *head,
                         if(add_partitions(part_head, 1, options) != 0) {
                             fprintf(stderr, "%s(): cannot create partition\n",
                                 __func__);
+                            *part_head = start_partition;
                             return (num_parts_created);
                         }
                         num_parts_created++;
@@ -261,7 +264,7 @@ dispatch_file_entries_by_limits(struct file_entry *head,
                     (*part_head)->size += head->size;
                     (*part_head)->num_files++;
 #if defined(DEBUG)
-                    fprintf(stderr, "%s(): %s added to partition %ju (%p)\n",
+                    fprintf(stderr, "%s(): %s assigned to partition_index %ju (%p)\n",
                         __func__, head->path, head->partition_index,
                         *part_head);
 #endif
@@ -281,5 +284,17 @@ dispatch_file_entries_by_limits(struct file_entry *head,
         current_partition_index = start_partition_index;
         *part_head = start_partition;
     }
+
+    /* empty partition cleanup:
+       When using option -s, if only special partition '0' (default_partition)
+       has been populated, remove start_partition to avoid returning an
+       additional -empty- partition */
+    if((max_size > 0) && (start_partition->num_files == 0)) {
+        remove_partition(start_partition);
+        start_partition = NULL;
+        num_parts_created--;
+        *part_head = default_partition;
+    }
+
     return (num_parts_created);
 }
