@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,6 +27,7 @@
 #include "types.h"
 #include "utils.h"
 #include "options.h"
+#include "partition.h"
 #include "file_entry.h"
 
 /* stat(2) */
@@ -92,6 +93,8 @@ static struct {
     pnum_t partition_index;      /* current partition number */
     fsize_t partition_size;      /* current partition size */
     fnum_t partition_num_files;  /* number of files in current partition */
+    int partition_errno;         /* 0 if every single entry has been fts_read()
+                                    without error, else last entry's errno */
     int exit_summary;            /* 0 if every single hook exit()ed with 0,
                                     else 1 */
     pid_t child_pid;
@@ -102,6 +105,7 @@ static struct {
     0,
     0,
     0,
+    0,
     -1
 };
 
@@ -126,10 +130,12 @@ kill_child(int sig)
 int
 fpart_hook(const char *cmd, const struct program_options *options,
     const char *live_filename, const pnum_t *live_partition_index,
-    const fsize_t *live_partition_size, const fnum_t *live_num_files)
+    const fsize_t *live_partition_size, const fnum_t *live_num_files,
+    const int partition_errno)
 {
     assert(cmd != NULL);
     assert(options != NULL);
+    assert(partition_errno >= 0);
 
     int retval = 0;
 
@@ -139,6 +145,7 @@ fpart_hook(const char *cmd, const struct program_options *options,
     char *env_fpart_partnumber_name = "FPART_PARTNUMBER";
     char *env_fpart_partsize_name = "FPART_PARTSIZE";
     char *env_fpart_partnumfiles_name = "FPART_PARTNUMFILES";
+    char *env_fpart_parterrno_name = "FPART_PARTERRNO";
     char *env_fpart_pid_name = "FPART_PID";
 
     /* env variables' values */
@@ -147,6 +154,7 @@ fpart_hook(const char *cmd, const struct program_options *options,
     char *env_fpart_partnumber_string = NULL;
     char *env_fpart_partsize_string = NULL;
     char *env_fpart_partnumfiles_string = NULL;
+    char *env_fpart_parterrno_string = NULL;
     char *env_fpart_pid_string = NULL;
 
     /* XXX As setenv(3)/unsetenv(3) are not available on all platforms, and there does not
@@ -169,7 +177,7 @@ fpart_hook(const char *cmd, const struct program_options *options,
 
         if(options->verbose >= OPT_VERBOSE)
             fprintf(stderr, "Executing pre-part #%ju hook: '%s'\n",
-                *live_partition_index, cmd);
+                adapt_partition_index(*live_partition_index, options), cmd);
 
         /* FPART_HOOKTYPE (pre-part) */
         malloc_size = strlen(env_fpart_hooktype_name) + 1 +
@@ -192,7 +200,7 @@ fpart_hook(const char *cmd, const struct program_options *options,
 
         if(options->verbose >= OPT_VERBOSE)
             fprintf(stderr, "Executing post-part #%ju hook: '%s'\n",
-                *live_partition_index, cmd);
+                adapt_partition_index(*live_partition_index, options), cmd);
 
         /* FPART_HOOKTYPE (post-part) */
         malloc_size = strlen(env_fpart_hooktype_name) + 1 +
@@ -228,13 +236,13 @@ fpart_hook(const char *cmd, const struct program_options *options,
     /* FPART_PARTNUMBER */
     if(live_partition_index != NULL) {
         malloc_size = strlen(env_fpart_partnumber_name) + 1 +
-            get_num_digits(*live_partition_index) + 1;
+            get_num_digits(adapt_partition_index(*live_partition_index, options)) + 1;
         if_not_malloc(env_fpart_partnumber_string, malloc_size,
             retval = 1;
             goto cleanup;
         )
         snprintf(env_fpart_partnumber_string, malloc_size, "%s=%ju",
-            env_fpart_partnumber_name, *live_partition_index);
+            env_fpart_partnumber_name, adapt_partition_index(*live_partition_index, options));
         if(push_env(env_fpart_partnumber_string, &envp) != 0) {
             retval = 1;
             goto cleanup;
@@ -273,6 +281,20 @@ fpart_hook(const char *cmd, const struct program_options *options,
         }
     }
 
+    /* FPART_PARTERRNO */
+    malloc_size = strlen(env_fpart_parterrno_name) + 1 +
+        get_num_digits(partition_errno) + 1;
+    if_not_malloc(env_fpart_parterrno_string, malloc_size,
+        retval = 1;
+        goto cleanup;
+    )
+    snprintf(env_fpart_parterrno_string, malloc_size, "%s=%d",
+        env_fpart_parterrno_name, partition_errno);
+    if(push_env(env_fpart_parterrno_string, &envp) != 0) {
+        retval = 1;
+        goto cleanup;
+    }
+
     /* FPART_PID */
     pid_t fpart_pid = getpid();
     malloc_size = strlen(env_fpart_pid_name) + 1 +
@@ -370,33 +392,77 @@ cleanup:
     return (retval);
 }
 
-/* Print or add a file entry (redirector) */
+/* Print or add a file entry (redirector)
+   - returns (0) if entry has been added
+   - returns (1) if entry has been skipped (option -S)
+   - returns (-1) if error */
 int
 handle_file_entry(struct file_entry **head, char *path, fsize_t size,
-    struct program_options *options)
+    int entry_errno, struct program_options *options)
 {
     assert(options != NULL);
+    assert(entry_errno >= 0);
 
     if(options->live_mode == OPT_LIVEMODE)
-        return (live_print_file_entry(path, size, options));
+        return (live_print_file_entry(path, size, entry_errno, options));
     else
+        /* XXX propagate (and exploit) entry_errno in non-live mode too ? */
         return (add_file_entry(head, path, size, options));
 }
 
-/* Print a file entry */
+/* Display a single entry line */
+void
+display_file_entry(pnum_t partition_index, const fsize_t entry_size,
+    const char * const entry_path, const unsigned char entry_display_type)
+{
+    assert(entry_path != NULL);
+
+    if(entry_display_type == ENTRY_DISPLAY_TYPE_STANDARD)
+        fprintf(stdout, "%ju\t%ju\t%s\n", partition_index, entry_size, entry_path);
+    else
+        fprintf(stdout, "S\t%ju\t%s\n", entry_size, entry_path);
+
+    return;
+}
+
+/* Print a file entry
+   - returns (0) if entry has been added
+   - returns (1) if entry has been skipped (option -S)
+   - returns (-1) if error */
 int
-live_print_file_entry(char *path, fsize_t size,
+live_print_file_entry(char *path, fsize_t size, int entry_errno,
     struct program_options *options)
 {
+/* split states */
+#define SPLIT_NONE 0
+#define SPLIT_DO   1
+#define SPLIT_END  2
+
     assert(path != NULL);
     assert(options != NULL);
     assert(options->live_mode == OPT_LIVEMODE);
+    assert(entry_errno >= 0);
 
     char *out_template = options->out_filename;
     char *ln_term = (options->out_zero == OPT_OUT0) ? "\0" : "\n";
+    int split = SPLIT_NONE;
+
+    /* option -S: skip files bigger than maximum partition size (option -s)
+       and print them to stdout in hardcoded pseudo-partition 'S' ('S'kipped).
+       Preloading and overloading are used here too */
+    if(options->skip_big == OPT_SKIPBIG) {
+        fsize_t needed_part_size = options->preload_size +
+            round_num(size + options->overload_size, options->round_size);
+        if(needed_part_size > options->max_size) {
+            display_file_entry(0, size, path, ENTRY_DISPLAY_TYPE_SKIPPED); /* partition_index irrelevant here */
+            fflush(stdout);
+            return (1);
+        }
+    }
 
     /* beginning of a new partition */
     if(live_status.partition_num_files == 0) {
+start_part:
         /* very first pass of first partition, preload first partition */
         if(live_status.partition_index == 0)
             live_status.partition_size = options->preload_size;
@@ -404,19 +470,22 @@ live_print_file_entry(char *path, fsize_t size,
         if(out_template != NULL) {
             /* compute live_status.filename "out_template.i\0" */
             size_t malloc_size = strlen(out_template) + 1 +
-                get_num_digits(live_status.partition_index) + 1;
+                get_num_digits(adapt_partition_index(live_status.partition_index, options)) + 1;
             if_not_malloc(live_status.filename, malloc_size,
-                return (1);
+                return (-1);
             )
             snprintf(live_status.filename, malloc_size, "%s.%ju", out_template,
-                live_status.partition_index);
+                adapt_partition_index(live_status.partition_index, options));
         }
 
         /* execute pre-partition hook */
         if(options->pre_part_hook != NULL) {
-            if(fpart_hook(options->pre_part_hook, options, live_status.filename,
-                &live_status.partition_index, &live_status.partition_size,
-                &live_status.partition_num_files) != 0)
+            if(fpart_hook(options->pre_part_hook, options,
+                live_status.filename,
+                &live_status.partition_index,
+                &live_status.partition_size,
+                &live_status.partition_num_files,
+                0) != 0) /* partition_errno irrelevant here */
                 live_status.exit_summary = 1;
         }
 
@@ -428,20 +497,34 @@ live_print_file_entry(char *path, fsize_t size,
                     strerror(errno));
                 free(live_status.filename);
                 live_status.filename = NULL;
-                return (1);
+                return (-1);
             }
         }
     }
 
+    if((options->dnr_split == OPT_DNRSPLIT) && (entry_errno != 0)) {
+        if (live_status.partition_num_files > 0) {
+            /* close current -non-empty- partition, do not (yet) count file in
+               and create a new partition to add our single -erroneous- entry */
+            split = SPLIT_DO;
+            goto end_part;
+        }
+        else
+            /* work on current partition */
+            split = SPLIT_END;
+    }
+
     /* count file in */
     live_status.partition_size +=
         round_num(size + options->overload_size, options->round_size);
     live_status.partition_num_files++;
+    if(entry_errno != 0)
+        live_status.partition_errno = entry_errno;
 
     if(out_template == NULL) {
-        /* no template provided, just print to stdout */
-        fprintf(stdout, "%ju (%ju): %s\n", live_status.partition_index, size,
-            path);
+        /* no template provided, just print entry to stdout */
+        display_file_entry(adapt_partition_index(live_status.partition_index, options),
+            size, path, ENTRY_DISPLAY_TYPE_STANDARD);
     }
     else {
         /* print to fd */
@@ -451,7 +534,7 @@ live_print_file_entry(char *path, fsize_t size,
             fprintf(stderr, "%s\n", strerror(errno));
             /* do not close(livefd) and free(live_status.filename) here because
                it will be useful and free'd in uninit_file_entries() below */
-            return (1);
+            return (-1);
         }
     }
 
@@ -460,15 +543,17 @@ live_print_file_entry(char *path, fsize_t size,
         fprintf(stderr, "%s\n", path);
 
     /* if end of partition reached */
-    if(((options->max_entries > 0) && 
+    if(((options->max_entries > 0) &&
             (live_status.partition_num_files >= options->max_entries)) ||
         ((options->max_size > 0) && 
-            (live_status.partition_size >= options->max_size))) {
+            (live_status.partition_size >= options->max_size)) ||
+        (split == SPLIT_END)) {
+end_part:
         /* display added partition */
         if(options->verbose >= OPT_VERBOSE)
-            fprintf(stderr, "Filled part #%ju: size = %ju, %ju file(s)\n",
-                live_status.partition_index, live_status.partition_size,
-                live_status.partition_num_files);
+            display_partition_summary(adapt_partition_index(live_status.partition_index, options),
+                live_status.partition_size, live_status.partition_num_files,
+                live_status.partition_errno, PARTITION_DISPLAY_TYPE_ERRNO);
 
         /* close fd or flush buffer */
         if(out_template == NULL)
@@ -479,9 +564,11 @@ live_print_file_entry(char *path, fsize_t size,
         /* execute post-partition hook */
         if(options->post_part_hook != NULL) {
             if(fpart_hook(options->post_part_hook, options,
-                live_status.filename, &live_status.partition_index,
+                live_status.filename,
+                &live_status.partition_index,
                 &live_status.partition_size,
-                &live_status.partition_num_files) != 0)
+                &live_status.partition_num_files,
+                live_status.partition_errno) != 0)
                 live_status.exit_summary = 1;
         }
 
@@ -494,6 +581,13 @@ live_print_file_entry(char *path, fsize_t size,
         live_status.partition_index++;
         live_status.partition_size = options->preload_size;
         live_status.partition_num_files = 0;
+        live_status.partition_errno = 0;
+
+        /* create another partition with single entry */
+        if (split == SPLIT_DO) {
+            split = SPLIT_END;
+            goto start_part;
+        }
     }
 
     return (0);
@@ -506,6 +600,8 @@ live_print_file_entry(char *path, fsize_t size,
 /* Add a file entry to a double-linked list of file_entries
    - if head is NULL, creates a new file entry ; if not, chains a new file
      entry to it
+   - returns (0) if entry has been added
+   - returns (-1) if error
    - returns with head set to the newly added element */
 int
 add_file_entry(struct file_entry **head, char *path, fsize_t size,
@@ -523,7 +619,7 @@ add_file_entry(struct file_entry **head, char *path, fsize_t size,
     previous = *current;
 
     if_not_malloc(*current, sizeof(struct file_entry),
-        return (1);
+        return (-1);
     )
 
     /* set head on first call */
@@ -536,7 +632,7 @@ add_file_entry(struct file_entry **head, char *path, fsize_t size,
     if_not_malloc((*current)->path, malloc_size,
         free(*current);
         *current = previous;
-        return (1);
+        return (-1);
     )
     snprintf((*current)->path, malloc_size, "%s", path);
     (*current)->size = size + options->overload_size;
@@ -642,24 +738,69 @@ init_file_entries(char *file_path, struct file_entry **head, fnum_t *count,
     unsigned char curdir_addme = 0;     /* current dir must be added */
     fsize_t curdir_size = 0;            /* current dir size */
 
+    int fts_read_errno = 0;             /* kept apart because p->fts_errno
+                                           is only significant with
+                                           FTS_ERR, FTS_DNR and FTS_NS
+                                           (see fts(3)), but fts_read_errno is
+                                           needed in FTS_DP */
+
     while((p = fts_read(ftsp)) != NULL) {
+        if(options->verbose >= OPT_VVVERBOSE) {
+            fprintf(stderr, "%s(%s): fts_info=%d, fts_errno=%d\n", __func__,
+                p->fts_path, p->fts_info, p->fts_errno);
+        }
+
+        /* update fts_read_errno with relevant values only */
         switch (p->fts_info) {
-            /* misc errors */
             case FTS_ERR:
+            case FTS_DNR:
+            case FTS_NS:
+                fts_read_errno = p->fts_errno;
+                break;
+            default:
+                fts_read_errno = 0;
+                break;
+        }
+
+        switch (p->fts_info) {
+            case FTS_ERR:   /* misc errors, but also partially-read dirs */
                 fprintf(stderr, "%s: %s\n", p->fts_path,
-                    strerror(p->fts_errno));
-                continue;
+                    strerror(fts_read_errno));
+
+                if(!S_ISDIR(p->fts_statp->st_mode))
+                    continue;
+                /* else, fallthrough to FTS_DNR for directory-related errors
+                   as fts_read() may return FTS_ERR during an un-finished
+                   readdir().
+
+                   Note: we may be adding parent directory entries while
+                   children files have previously been packed, e.g.:
+                     Partition 1 containing /tmp/foo/file1
+                     [readdir() error while reading /tmp/foo/]
+                     Partition 2 containing /tmp/foo/ + errno set
+                   Do we need an option to enable that fallthrough here ?
+                   Or provide a way to consumer to differentiate FTS_ERR from
+                   FTS_DNR ?
+                */
 
             /* errors for which we know there is a file or directory
                within current directory */
-            case FTS_DNR:   /* un-readable directory */
+            case FTS_DNR:   /* un-readable directory, which can be:
+                               - because of EACCES (permission denied)
+                               - because of a readdir() *technical* error
+                                 with directory not read at all
+                               - because of a readdir() *technical* error
+                                 with directory partially-read
+                                 (because of FTS_ERR fallthrough above) */
             {
                 fprintf(stderr, "%s: %s\n", p->fts_path,
-                    strerror(p->fts_errno));
+                    strerror(fts_read_errno));
+
                 /* if requested by the -zz option,
                    add directory anyway by simulating FTS_DP */
                 if(options->dirs_include >= OPT_DNREMPTY) {
-                    curdir_empty = 1;
+                    curdir_empty = 1; /* avoid re-crawling erroneous dir
+                                         through get_size() */
                     goto add_directory;
                 }
                 /* else, mark current dir as not empty */
@@ -669,15 +810,15 @@ init_file_entries(char *file_path, struct file_entry **head, fnum_t *count,
             }
             case FTS_NS:    /* stat() error */
                 fprintf(stderr, "%s: %s\n", p->fts_path,
-                    strerror(p->fts_errno));
-            case FTS_NSOK: /* no stat(2) available (not requested) */
+                    strerror(fts_read_errno));
+            case FTS_NSOK:  /* no stat(2) available (not requested) */
                 /* mark current dir as not empty */
                 curdir_empty = 0;
                 continue;
 
             case FTS_DC:
                 fprintf(stderr, "%s: filesystem loop detected\n", p->fts_path);
-            case FTS_DOT:  /* ignore "." and ".." */
+            case FTS_DOT:   /* ignore "." and ".." */
                 continue;
 
             case FTS_DP:
@@ -755,10 +896,11 @@ add_directory:
                     /* else, trust curdir_size and leave it untouched. */
 
                     /* add or display it */
-                    if(handle_file_entry
-                        (head, curdir_entry_path, curdir_size, options) == 0)
+                    int handled = handle_file_entry(head, curdir_entry_path,
+                        curdir_size, fts_read_errno, options);
+                    if(handled == 0)
                         (*count)++;
-                    else {
+                    else if(handled < 0) {
                         fprintf(stderr, "%s(): cannot add file entry\n",
                             __func__);
                         free(curdir_entry_path);
@@ -808,9 +950,8 @@ reset_directory:
                 continue;
             }
 
-            default:
-            /* XXX default means remaining file types:
-               FTS_F, FTS_SL, FTS_SLNONE, FTS_DEFAULT */
+            default:    /* XXX default means remaining file types:
+                           FTS_F, FTS_SL, FTS_SLNONE, FTS_DEFAULT */
             {
                 /* get current file size to add it to our current directory
                    size. We must have visited all directories first for that
@@ -867,10 +1008,12 @@ two-pass check to handle include and exclude options properly. */
                     continue;
 
                 /* add or display it */
-                if(handle_file_entry
-                    (head, p->fts_path, curfile_size, options) == 0)
+                int handled = handle_file_entry(head, p->fts_path,
+                    curfile_size, 0, options); /* fts_read_errno is always 0
+                                                  here, so harcode it */
+                if(handled == 0)
                     (*count)++;
-                else {
+                else if(handled < 0) {
                     fprintf(stderr, "%s(): cannot add file entry\n", __func__);
                     fts_close(ftsp);
                     return (1);
@@ -918,9 +1061,9 @@ uninit_file_entries(struct file_entry *head, struct program_options *options)
         /* display added partition */
         if((options->verbose >= OPT_VERBOSE) &&
             (live_status.partition_num_files > 0))
-            fprintf(stderr, "Filled part #%ju: size = %ju, %ju file(s)\n",
-                live_status.partition_index, live_status.partition_size,
-                live_status.partition_num_files);
+            display_partition_summary(adapt_partition_index(live_status.partition_index, options),
+                live_status.partition_size, live_status.partition_num_files,
+                live_status.partition_errno, PARTITION_DISPLAY_TYPE_ERRNO);
 
         /* flush buffer or close last file if necessary */
         if(options->out_filename == NULL)
@@ -932,9 +1075,11 @@ uninit_file_entries(struct file_entry *head, struct program_options *options)
         if((options->post_part_hook != NULL) &&
             (live_status.partition_num_files > 0)) {
             if(fpart_hook(options->post_part_hook, options,
-                live_status.filename, &live_status.partition_index,
+                live_status.filename,
+                &live_status.partition_index,
                 &live_status.partition_size,
-                &live_status.partition_num_files) != 0)
+                &live_status.partition_num_files,
+                live_status.partition_errno) != 0)
                 live_status.exit_summary = 1;
         }
 
@@ -954,10 +1099,11 @@ uninit_file_entries(struct file_entry *head, struct program_options *options)
 /* Print a double-linked list of file_entries from head
    - if no filename template given, print to stdout */
 int
-print_file_entries(struct file_entry *head, pnum_t num_parts,
-    struct program_options *options)
+print_file_entries(struct file_entry *head, struct partition *part_head,
+    pnum_t num_parts, struct program_options *options)
 {
     assert(head != NULL);
+    assert(part_head != NULL);
     assert(num_parts > 0);
     assert(options != NULL);
 
@@ -967,8 +1113,8 @@ print_file_entries(struct file_entry *head, pnum_t num_parts,
     /* no template provided, just print to stdout and return */
     if(out_template == NULL) {
         while(head != NULL) {
-            fprintf(stdout, "%ju (%ju): %s\n", head->partition_index,
-                head->size, head->path);
+            display_file_entry(adapt_partition_index(head->partition_index, options),
+                head->size, head->path, ENTRY_DISPLAY_TYPE_STANDARD);
             head = head->nextp;
         }
         return (0);
@@ -976,48 +1122,67 @@ print_file_entries(struct file_entry *head, pnum_t num_parts,
 
     /* a template has been provided; to avoid opening too many files,
        open chunks of FDs and do as many passes as necessary */
-    struct file_entry *start = head;
-    pnum_t current_chunk = 0;           /* current chunk */
-    pnum_t current_file_entry = 0;      /* current file entry within chunk */
-
     assert(PRINT_FE_CHUNKS > 0);
 
-    while(((current_chunk * PRINT_FE_CHUNKS) + current_file_entry) <
-        num_parts) {
+    struct file_entry *start = head;
+    pnum_t current_chunk = 0;       /* current chunk */
+    pnum_t current_fd_index = 0;    /* current file index within chunk */
+
+    /* current -absolute- partition index */
+#define current_partition_index \
+    ((current_chunk * PRINT_FE_CHUNKS) + current_fd_index)
+
+    /* current_partition_index gets incremented by PRINT_FE_CHUNKS */
+    while(current_partition_index < num_parts) {
         int fd[PRINT_FE_CHUNKS]; /* our <files per chunk> file descriptors */
 
         /* open as necessary file descriptors as needed
            to print num_part partitions */
-        while((current_file_entry < PRINT_FE_CHUNKS) &&
-              (((current_chunk * PRINT_FE_CHUNKS) + current_file_entry) < num_parts)) {
+        while((current_fd_index < PRINT_FE_CHUNKS) &&
+              (current_partition_index < num_parts)) {
+            /* skip empty partition '0':
+               XXX No need to skip write() calls below, as no file entry should
+               be associated with that partition */
+            if((current_partition_index == 0) && (part_head->num_files == 0)) {
+#if defined(DEBUG)
+                fprintf(stderr, "%s(): skip creating empty partition '0'\n",
+                    __func__);
+#endif
+                fd[current_fd_index] = (-1); /* for close() calls below */
+                current_fd_index++;
+                continue;
+            }
+
             /* compute out_filename  "out_template.i\0" */
             char *out_filename = NULL;
             size_t malloc_size = strlen(out_template) + 1 + get_num_digits
-                ((current_chunk * PRINT_FE_CHUNKS) + current_file_entry) + 1;
+                (adapt_partition_index(current_partition_index, options)) + 1;
             if_not_malloc(out_filename, malloc_size,
                 /* close all open descriptors and return */
                 pnum_t i;
-                for(i = 0; i < current_file_entry; i++)
+                for(i = 0; i < current_fd_index; i++)
                      close(fd[i]);
                 return (1);
             )
             snprintf(out_filename, malloc_size, "%s.%ju", out_template,
-                (current_chunk * PRINT_FE_CHUNKS) + current_file_entry);
+                adapt_partition_index(current_partition_index, options));
 
-            if((fd[current_file_entry] =
+            if((fd[current_fd_index] =
                 open(out_filename, O_WRONLY|O_CREAT|O_TRUNC, 0660)) < 0) {
                 fprintf(stderr, "%s: %s\n", out_filename, strerror(errno));
                 free(out_filename);
                 /* close all open descriptors and return */
                 pnum_t i;
-                for(i = 0; i < current_file_entry; i++)
+                for(i = 0; i < current_fd_index; i++)
                      close(fd[i]);
                 return (1);
             }
             free(out_filename);
-            current_file_entry++;
+
+            current_fd_index++;
         }
 
+        /* write data to opened file descriptors */
         while(head != NULL) {
             if((head->partition_index >= (current_chunk * PRINT_FE_CHUNKS)) &&
                (head->partition_index < ((current_chunk + 1) * PRINT_FE_CHUNKS))) {
@@ -1042,7 +1207,7 @@ print_file_entries(struct file_entry *head, pnum_t num_parts,
         for(i = 0; (i < PRINT_FE_CHUNKS) && (((current_chunk * PRINT_FE_CHUNKS) + i) < num_parts); i++)
             close(fd[i]);
 
-        current_file_entry = 0;
+        current_fd_index = 0;
         current_chunk++;
     }
 
