@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,10 +50,16 @@ struct file_entry {
 
 int fpart_hook(const char *cmd, const struct program_options *options,
     const char *live_filename, const pnum_t *live_partition_index,
-    const fsize_t *live_partition_size, const fnum_t *live_num_files);
+    const fsize_t *live_partition_size, const fnum_t *live_num_files,
+    const int partition_errno);
 int handle_file_entry(struct file_entry **head, char *path, fsize_t size,
-    struct program_options *options);
-int live_print_file_entry(char *path, fsize_t size,
+    int entry_errno, struct program_options *options);
+/* display types */
+#define ENTRY_DISPLAY_TYPE_STANDARD 0
+#define ENTRY_DISPLAY_TYPE_SKIPPED  1
+void display_file_entry(pnum_t partition_index, const fsize_t entry_size,
+    const char * const entry_path, const unsigned char entry_display_type);
+int live_print_file_entry(char *path, fsize_t size, int entry_errno,
     struct program_options *options);
 int add_file_entry(struct file_entry **head, char *path, fsize_t size,
     struct program_options *options);
@@ -61,8 +67,8 @@ int init_file_entries(char *file_path, struct file_entry **head, fnum_t *count,
     struct program_options *options);
 void uninit_file_entries(struct file_entry *head,
     struct program_options *options);
-int print_file_entries(struct file_entry *head, pnum_t num_parts,
-    struct program_options *options);
+int print_file_entries(struct file_entry *head, struct partition *part_head,
+    pnum_t num_parts, struct program_options *options);
 void init_file_entry_p(struct file_entry **file_entry_p, fnum_t num_entries,
     struct file_entry *head);
 
