@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,7 @@ static void
 version(void)
 {
     fprintf(stderr, "fpart v" FPART_VERSION "\n"
-        "Copyright (c) 2011-2021 "
+        "Copyright (c) 2011-2022 "
         "Ganael LAPLANCHE <ganael.laplanche@martymac.org>\n"
         "WWW: http://contribs.martymac.org\n");
     fprintf(stderr, "Build options: debug=");
@@ -132,6 +132,8 @@ usage(void)
         "(default: pack files only)\n");
     fprintf(stderr, "  -zz\ttreat un-readable directories as empty\n");
     fprintf(stderr, "  -zzz\tpack all directories (as empty)\n");
+    fprintf(stderr, "  -Z\tpack un-readable directories in separate "
+        "partitions\n");
     fprintf(stderr, "  -d\tpack directories instead of files after a certain "
         "<depth>\n");
     fprintf(stderr, "  -D\tpack leaf directories (i.e. containing files only, "
@@ -141,6 +143,10 @@ usage(void)
     fprintf(stderr, "Live mode:\n");
     fprintf(stderr, "  -L\tlive mode: generate partitions during filesystem "
         "crawling\n");
+    fprintf(stderr, "  -S\tdo not pack files bigger than specified maximum "
+        "partition size\n");
+    fprintf(stderr, "    \tbut print them to stdout instead (needs -L and "
+        "-s)\n");
     fprintf(stderr, "  -w\tpre-partition hook: execute <cmd> at partition "
         "start\n");
     fprintf(stderr, "  -W\tpost-partition hook: execute <cmd> at partition "
@@ -182,9 +188,11 @@ handle_argument(char *argument, fnum_t *totalfiles, struct file_entry **head,
         )
 
         if(sscanf(argument, "%ju %[^\n]", &input_size, input_path) == 2) {
-            if(handle_file_entry(head, input_path, input_size, options) == 0)
+            int handled = handle_file_entry(head, input_path, input_size,
+                0, options); /* entry_errno irrelevant here */
+            if(handled == 0)
                 (*totalfiles)++;
-            else {
+            else if(handled < 0) {
                 fprintf(stderr, "%s(): cannot add file entry\n", __func__);
                 free(input_path);
                 return (1);
@@ -260,9 +268,9 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
     int ch;
     while((ch = getopt(*argcp, *argvp,
 #if defined(_HAS_FNM_CASEFOLD)
-        "hVn:f:s:i:ao:0evlby:Y:x:X:zd:DELw:W:p:q:r:"
+        "hVn:f:s:i:ao:0evlby:Y:x:X:zZd:DELSw:W:p:q:r:"
 #else
-        "hVn:f:s:i:ao:0evlby:x:zd:DELw:W:p:q:r:"
+        "hVn:f:s:i:ao:0evlby:x:zZd:DELSw:W:p:q:r:"
 #endif
         )) != -1) {
         switch(ch) {
@@ -397,6 +405,9 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
             case 'z':
                 options->dirs_include++;
                 break;
+            case 'Z':
+                options->dnr_split = OPT_DNRSPLIT;
+                break;
             case 'd':
             {
                 char *endptr = NULL;
@@ -418,6 +429,9 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
             case 'L':
                 options->live_mode = OPT_LIVEMODE;
                 break;
+            case 'S':
+                options->skip_big = OPT_SKIPBIG;
+                break;
             case 'w':
             {
                 /* check for empty argument */
@@ -556,6 +570,24 @@ handle_options(struct program_options *options, int *argcp, char ***argvp)
         return (FPART_OPTS_USAGE | FPART_OPTS_NOK | FPART_OPTS_EXIT);
     }
 
+    /* Option -Z requires -L and -zz (or -zzz) */
+    if((options->dnr_split == OPT_DNRSPLIT) &&
+        ((options->live_mode == OPT_NOLIVEMODE) ||
+         (options->dirs_include < OPT_DNREMPTY))) {
+        fprintf(stderr,
+            "Option -Z requires options -L and -zz (or -zzz).\n");
+        return (FPART_OPTS_USAGE | FPART_OPTS_NOK | FPART_OPTS_EXIT);
+    }
+
+    /* option -S (needs '-L' and '-s') */
+    if((options->skip_big == OPT_SKIPBIG) &&
+        ((options->live_mode == OPT_NOLIVEMODE) ||
+        (options->max_size == DFLT_OPT_MAX_SIZE))) {
+        fprintf(stderr,
+            "Option -S can only be used with options -L and -s.\n");
+        return (FPART_OPTS_USAGE | FPART_OPTS_NOK | FPART_OPTS_EXIT);
+    }
+
     if((options->in_filename == NULL) && (*argcp <= 0)) {
         /* no file specified, force stdin */
         char *opt_input = "-";
@@ -631,7 +663,7 @@ int main(int argc, char **argv)
         bzero(line, MAX_LINE_LENGTH);
         while(fgets(line, MAX_LINE_LENGTH, in_fp) != NULL) {
             /* replace '\n' with '\0' */
-            if ((line_end_p = strchr(line, '\n')) != NULL)
+            if((line_end_p = strchr(line, '\n')) != NULL)
                 *line_end_p = '\0';
 
             if(handle_argument(line, &totalfiles, &head, &options) != 0) {
@@ -787,13 +819,13 @@ int main(int argc, char **argv)
 ************************/
 
     /* print result summary */
-    print_partitions(part_head);
+    print_partitions(part_head, &options);
 
     if(options.verbose >= OPT_VERBOSE)
         fprintf(stderr, "Writing output lists...\n");
 
     /* print file entries */
-    print_file_entries(head, num_parts, &options);
+    print_file_entries(head, part_head, num_parts, &options);
 
     if(options.verbose >= OPT_VERBOSE)
         fprintf(stderr, "Cleaning up...\n");
