@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,6 +62,8 @@ init_options(struct program_options *options)
            (DFLT_OPT_DIRSINCLUDE == OPT_EMPTYDIRS) ||
            (DFLT_OPT_DIRSINCLUDE == OPT_DNREMPTY) ||
            (DFLT_OPT_DIRSINCLUDE == OPT_ALLDIRS));
+    assert((DFLT_OPT_DNRSPLIT == OPT_NODNRSPLIT) ||
+           (DFLT_OPT_DNRSPLIT == OPT_DNRSPLIT));
     assert(DFLT_OPT_DIR_DEPTH >= OPT_NODIRDEPTH);
     assert((DFLT_OPT_LEAFDIRS == OPT_NOLEAFDIRS) ||
            (DFLT_OPT_LEAFDIRS == OPT_LEAFDIRS));
@@ -69,6 +71,8 @@ init_options(struct program_options *options)
            (DFLT_OPT_DIRSONLY == OPT_DIRSONLY));
     assert((DFLT_OPT_LIVEMODE == OPT_NOLIVEMODE) ||
            (DFLT_OPT_LIVEMODE == OPT_LIVEMODE));
+    assert((DFLT_OPT_SKIPBIG == OPT_NOSKIPBIG) ||
+           (DFLT_OPT_SKIPBIG == OPT_SKIPBIG));
     assert(DFLT_OPT_PRELOAD_SIZE >= 0);
     assert(DFLT_OPT_OVERLOAD_SIZE >= 0);
     assert(DFLT_OPT_ROUND_SIZE >= 1);
@@ -95,9 +99,11 @@ init_options(struct program_options *options)
     options->nexclude_files_ci = 0;
     options->dirs_include = DFLT_OPT_DIRSINCLUDE;
     options->dir_depth = DFLT_OPT_DIR_DEPTH;
+    options->dnr_split = OPT_NODNRSPLIT;
     options->leaf_dirs = DFLT_OPT_LEAFDIRS;
     options->dirs_only = DFLT_OPT_DIRSONLY;
     options->live_mode = DFLT_OPT_LIVEMODE;
+    options->skip_big = DFLT_OPT_SKIPBIG;
     options->pre_part_hook = NULL;
     options->post_part_hook = NULL;
     options->preload_size = DFLT_OPT_PRELOAD_SIZE;
@@ -116,9 +122,11 @@ uninit_options(struct program_options *options)
         free(options->post_part_hook);
     if(options->pre_part_hook != NULL)
         free(options->pre_part_hook);
+    options->skip_big = DFLT_OPT_SKIPBIG;
     options->live_mode = DFLT_OPT_LIVEMODE;
     options->dirs_only = DFLT_OPT_DIRSONLY;
     options->leaf_dirs = DFLT_OPT_LEAFDIRS;
+    options->dnr_split = OPT_NODNRSPLIT;
     options->dir_depth = DFLT_OPT_DIR_DEPTH;
     options->dirs_include = DFLT_OPT_DIRSINCLUDE;
     if(options->exclude_files_ci != NULL)
