@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -69,6 +69,7 @@ struct program_options {
 #define OPT_NOVERBOSE               0
 #define OPT_VERBOSE                 1
 #define OPT_VVERBOSE                2
+#define OPT_VVVERBOSE               3
 #define DFLT_OPT_VERBOSE            OPT_NOVERBOSE
     unsigned char verbose;
 /* follow symbolic links (option -l) */
@@ -100,6 +101,11 @@ struct program_options {
 #define OPT_ALLDIRS                 3   /* include all directories */
 #define DFLT_OPT_DIRSINCLUDE        OPT_NOEMPTYDIRS
     unsigned char dirs_include;
+/* pack un-readable directories in separate partitions (option -Z) */
+#define OPT_NODNRSPLIT              0
+#define OPT_DNRSPLIT                1
+#define DFLT_OPT_DNRSPLIT           OPT_NODNRSPLIT
+    unsigned char dnr_split;
 /* display directories after certain depth (option -d) */
 #define OPT_NODIRDEPTH              -1
 #define DFLT_OPT_DIR_DEPTH          OPT_NODIRDEPTH
@@ -119,6 +125,11 @@ struct program_options {
 #define OPT_LIVEMODE                1
 #define DFLT_OPT_LIVEMODE           OPT_NOLIVEMODE
     unsigned char live_mode;
+/* skip big files (option -S) */
+#define OPT_NOSKIPBIG            0
+#define OPT_SKIPBIG              1
+#define DFLT_OPT_SKIPBIG         OPT_NOSKIPBIG
+    unsigned char skip_big;
 /* pre-partition hook (option -w) */
     char *pre_part_hook;
 /* post-partition hook (option -W) */
