@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -84,6 +84,24 @@ add_partitions(struct partition **head, pnum_t num_parts,
     return (0);
 }
 
+/* Remove a specific partition from the chain */
+int
+remove_partition(struct partition *part)
+{
+    assert(part != NULL);
+
+    /* unlink partition */
+    if(part->prevp != NULL)
+            part->prevp->nextp = part->nextp;
+    if(part->nextp != NULL)
+            part->nextp->prevp = part->prevp;
+
+    /* free memory */
+    free(part);
+
+    return (0);
+}
+
 /* Un-initialize a double-linked list of partitions */
 void
 uninit_partitions(struct partition *head)
@@ -144,16 +162,57 @@ get_partition_at(struct partition *head, pnum_t index)
     return (head);
 }
 
+/* Adapt partition index for output, regarding program options
+   - returns an index suitable for user output (display or filename) */
+pnum_t
+adapt_partition_index(pnum_t index, const struct program_options *options)
+{
+    assert(options != NULL);
+
+    pnum_t offset = 1;
+
+    /* compute output index offset: fpart always produces partitions starting
+       from '1' (but they internally start from '0'). Partition '0' -used for
+       large files that do not fit in regular partitions- only appears when
+       option '-s' is passed *and* non-live mode is used */
+    if((options->max_size != DFLT_OPT_MAX_SIZE) &&
+       (options->live_mode == OPT_NOLIVEMODE))
+        offset = 0;
+
+    return (index + offset);
+}
+
+/* Display a partition summary */
+void
+display_partition_summary(pnum_t partition_index, const fsize_t partition_size,
+    const fnum_t partition_num_files, int partition_errno,
+    const unsigned char partition_display_type)
+{
+    if(partition_display_type == PARTITION_DISPLAY_TYPE_STANDARD)
+        fprintf(stderr, "Part #%ju: size = %ju, files = %ju\n",
+            partition_index, partition_size, partition_num_files);
+    else
+        fprintf(stderr, "Part #%ju: size = %ju, files = %ju, errno = %d\n",
+            partition_index, partition_size, partition_num_files, partition_errno);
+
+    return;
+}
+
 /* Print partitions from head */
 void
-print_partitions(struct partition *head)
+print_partitions(struct partition *head, struct program_options *options)
 {
-    pnum_t i = 0;
+    assert(options != NULL);
+
+    pnum_t partition_index = 0;
     while(head != NULL) {
-        fprintf(stderr, "Part #%ju: size = %ju, %ju file(s)\n", i,
-            head->size, head->num_files);
+        /* skip empty partition '0' */
+        if((partition_index != 0) || (head->num_files != 0))
+            display_partition_summary(adapt_partition_index(partition_index, options),
+                head->size, head->num_files, 0, /* partition_errno irrelevant here */
+                PARTITION_DISPLAY_TYPE_STANDARD);
         head = head->nextp;
-        i++;
+        partition_index++;
     }
     return;
 }
