@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,9 +44,16 @@ struct partition {
 
 int add_partitions(struct partition **head, pnum_t num_parts,
     struct program_options *options);
+int remove_partition(struct partition *part);
 void uninit_partitions(struct partition *head);
 pnum_t find_smallest_partition_index(struct partition *head);
 struct partition * get_partition_at(struct partition *head, pnum_t index);
-void print_partitions(struct partition *head);
+pnum_t adapt_partition_index(pnum_t index, const struct program_options *options);
+#define PARTITION_DISPLAY_TYPE_STANDARD 0
+#define PARTITION_DISPLAY_TYPE_ERRNO    1
+void display_partition_summary(pnum_t partition_index,
+    const fsize_t partition_size, const fnum_t partition_num_files,
+    int partition_errno, const unsigned char partition_display_type);
+void print_partitions(struct partition *head, struct program_options *options);
 
 #endif /* _PARTITION_H */
