@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2021 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2022 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -162,6 +162,10 @@ get_size(char *file_path, struct stat *file_stat,
     }
 
     while((p = fts_read(ftsp)) != NULL) {
+        if(options->verbose >= OPT_VVVERBOSE) {
+            fprintf(stderr, "%s(%s): fts_info=%d, ftp_errno=%d\n", __func__,
+                p->fts_path, p->fts_info, p->fts_errno);
+        }
         switch (p->fts_info) {
             case FTS_ERR:   /* misc error */
             case FTS_DNR:   /* un-readable directory */
@@ -184,10 +188,10 @@ get_size(char *file_path, struct stat *file_stat,
                    already been made before */
                 if((!valid_file(p, options, VF_EXCLUDEONLY)) &&
                     (p->fts_level > 0)) {
-#if defined(DEBUG)
-                    fprintf(stderr, "%s(): skipping directory: %s\n", __func__,
-                        p->fts_path);
-#endif
+                    if(options->verbose >= OPT_VVVERBOSE) {
+                        fprintf(stderr, "%s(): skipping directory: %s\n", __func__,
+                            p->fts_path);
+                    }
                     fts_set(ftsp, p, FTS_SKIP);
                 }
                 continue;
@@ -198,10 +202,10 @@ get_size(char *file_path, struct stat *file_stat,
 
                 /* Excluded files do not account for returned size */
                 if(!valid_file(p, options, VF_EXCLUDEONLY)) {
-#if defined(DEBUG)
-                    fprintf(stderr, "%s(): skipping file: %s\n", __func__,
-                        p->fts_path);
-#endif
+                    if(options->verbose >= OPT_VVVERBOSE) {
+                        fprintf(stderr, "%s(): skipping file: %s\n", __func__,
+                            p->fts_path);
+                    }
                 }
                 else
                     file_size += p->fts_statp->st_size;
@@ -376,7 +380,7 @@ str_to_uintmax(const char *str, const unsigned char handle_multiplier)
 
         uintmax_t orig_val = val;
         /* more than one character remain or invalid multiplier specified */
-        if ((*(endptr + 1) != '\0') ||
+        if((*(endptr + 1) != '\0') ||
             (multiplier = char_to_multiplier(*endptr)) == 0) {
             fprintf(stderr, "%s(): %s\n", __func__, "unknown unit provided");
             return (0);
